/*
 * Copyright 2009 OW2 Chameleon
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ow2.chameleon.core;

import java.io.File;
import java.net.MalformedURLException;

import junit.framework.TestCase;

import org.mockito.Mockito;
import org.osgi.framework.BundleContext;

/**
 * Debug Activator unit tests.
 * @author clement escoffier clement.escoffier@akquinet.de
 */
public class DebugActivatorTest extends TestCase {

    /**
     * Simulates the starting with the debug activator.
     * @throws MalformedURLException if something bad happened
     */
    public void testFullCreation() throws MalformedURLException {
        File mockedDir = MockUtils.getCompleteCoreDirectory();
        assertTrue(mockedDir.exists());
        assertTrue(mockedDir.isDirectory());
        DebugActivator activator = new DebugActivator(mockedDir);
        assertNotNull(activator);
    }

    /**
     * Simulates the starting with the debug activator.
     * Checks the installation.
     * @throws Exception if something bad happened
     */
    public void testFullStart() throws Exception {
        File mockedDir = MockUtils.getCompleteCoreDirectory();

        BundleContext mockedContext = MockUtils.getContext();
        DebugActivator activator = new DebugActivator(mockedDir);
        activator.start(mockedContext);

        // Verify
        // Three install
        ((BundleContext) Mockito.verify(mockedContext, Mockito.times(1)))
                .installBundle(Mockito.anyString());
    }

    /**
     * Simulates the starting with the debug activator
     * with an incomplete core.
     * @throws Exception if something bad happened
     */
    public void testIncompleteStart() throws Exception {
        File mockedDir = MockUtils.getIncompleteCoreDirectory();

        BundleContext mockedContext = MockUtils.getContext();
        DebugActivator activator = new DebugActivator(mockedDir);

        try {
            activator.start(mockedContext);
            fail("Incomplete directory undetected");
        } catch (Exception e) {
            // OK
        }

        // Verify
        // No install
        ((BundleContext) Mockito.verify(mockedContext, Mockito.never()))
                .installBundle(Mockito.anyString());

    }

    /**
     * Check stopping.
     * @throws Exception if something bad happened
     */
    public void testStop() throws Exception {
        File mockedDir = MockUtils.getCompleteCoreDirectory();

        BundleContext mockedContext = MockUtils.getContext();
        CoreActivator activator = new CoreActivator(mockedDir);
        activator.stop(mockedContext);
    }

    /**
     * Checks on real core.
     * @throws Exception if something bad happened
     */
    public void testOnFullCoreDirectory() throws Exception {
        File core = new File(Constants.FULL_CORE);

        BundleContext mockedContext = MockUtils.getContext();
        DebugActivator activator = new DebugActivator(core);
        activator.start(mockedContext);

        // Verify
        ((BundleContext) Mockito.verify(mockedContext, Mockito.times(1)))
                .installBundle(Mockito.anyString());
    }

    /**
     * Checks on an incomplete core.
     * @throws Exception if something bad happened
     */
    public void testOnIncompleteCoreDirectory() throws Exception {
        File core = new File(
                Constants.INCOMPLETE_CORE);

        BundleContext mockedContext = MockUtils.getContext();
        DebugActivator activator = new DebugActivator(core);

        try {
            activator.start(mockedContext);
            fail("Incomplete directory undetected");
        } catch (Exception e) {
            // OK
        }

        // Verify
        // No install
        ((BundleContext) Mockito.verify(mockedContext, Mockito.never()))
                .installBundle(Mockito.anyString());

    }

}
